import subprocess
import csv
import sys
import time
import os
import signal
import datetime
import requests
import json
import math
import random
from sb import isSafe



# ACQUIRE COMMAND LINE ARGUMENTS

# 1 - adb serial
# 2 - number of devices (to generate the right interval and calibrate the starting number)
# 3 - rank of the first website to visit
# 4 - device model  exploited
# 5 - csv file containing the domain to visit with the associated rank (<rank>,<domain>)

serial = str(sys.argv[1])

if int(sys.argv[2]) < 1:
    print("ERROR, number of devices must be greater than 0")
    exit()
else:    
    interval = int(sys.argv[2])-1

if int(sys.argv[3]) < 1:
    print("ERROR, starting number must be greater than 0")
    exit()
else:
    if int(sys.argv[3])%interval == 0:
        firstsite = (int(sys.argv[3])/4)-1
    else:
        firstsite = (int(sys.argv[3])/4) # it should be +1 and -1 = nothing

if str(sys.argv[4]) == 'opo':
    model = 'opo'
    x = 1080
    y = 1670
    nav = 0
elif str(sys.argv[4]) == 'nex':
    model = 'nex'
    x = 1080
    y = 1670
    nav = 140
else:
    print("ERROR, model not valid")
    exit()

path = str(sys.argv[5])

bashff = "adb -s " + serial + " shell am start -a android.intent.action.VIEW -n org.mozilla.firefox/org.mozilla.gecko.BrowserApp -d '"
rootadb = "adb -s " + serial + " root"
logcommand = "adb -s " + serial + " logcat -v time"
killfirefox = "adb -s " + serial + " shell su -c \'pkill firefox\'"


global firstrowlog
firstrowlog = "#%$^@#$%!#$@%#$TEFGWSEW IMPOSSIBLESTRING #$#^@#$T#R@Q#EFEASDAEWQ#CT$GFQ"

global domainold
domainold = None

# --- CREATE FILES AND LOG FOLDERS IF NOT EXISTING

i = 0
skip = interval
errlog = open('errorlog', 'a+', 1)
rollbacklog = open('rollbacklog', 'a+', 1) # logs of any redirection that required a rollback
loadingfailslog = open('loadingfailslog', 'a+', 1) # logs of incomplete or failing websites loading
unsafewebsites = open('unsafewebsites', 'a+', 1) # logs of websites considered malicious by SafeBrowsing API
failedgrantingpos = open('failedgrantingpos', 'a+', 1) # logs of websites accessing position but script failed granting it
failedgrantingvib = open('failedgrantingvib', 'a+', 1) # logs of websites accessing vibration but script failed granting it

subprocess.run(("mkdir jsfiles").split(" "))
subprocess.run(("mkdir logs"+serial).split(" "))



def getPidName(name):
    pid = subprocess.check_output(["pgrep", "-f",name])
    return int(pid)
    
def cleanLogcat():
    global firstrowlog
    subprocess.run(("adb -s "+serial+" shell su -c \'pkill logcat\'").split(" ")) # kills all processes using logcat to avoid cleaning race conditions
    subprocess.run(("adb -s "+serial+" logcat -b all -c").split(" "))
    p1 = subprocess.Popen(["adb", "-s", serial, "logcat", "-v", "time"], stdout=subprocess.PIPE)
    first = subprocess.check_output(["head", "-n", "2"], stdin=p1.stdout)
    p1.stdout.close()
    p1.terminate()
    if firstrowlog in str(first):
        return False
    else:
        firstrowlog = str(first)
        return True


def checkCurrentDomain():
    p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["grep", "-a","REDIRECTIONTO"], stdin=p1.stdout, stdout=subprocess.PIPE)
    url = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
    p1.stdout.close()
    p2.stdout.close()
    p1.terminate()
    p2.terminate()
    try:
        domain = str(url).split(": ")[2][:-3]
        if domain == domainold:
            errlog.write(str(datetime.datetime.now()) + "DOMAIN is the last one, not saved : ["+domainold+"]\n")
            print("----------> DOMAIN is the last one, not saved : ["+domainold+"] <------------\n")
            return "None"
        else:
            print("CURRENT DOMAIN --------------> " + str(url).split(": ")[2][:-3] + "<--------------------")
            return domain
    
    except:
        errlog.write(str(datetime.datetime.now()) +" "+ str(url) + "  [" + row[1]+"]\n")
        print("ERROR in REDIRECTION DETECTION (NOT FOUND)")
        return "None"

def checkPermissionRequests():
    p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["grep", "-a","MWEBAPILOGP;"], stdin=p1.stdout, stdout=subprocess.PIPE)
    permission = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
    p1.stdout.close()
    p2.stdout.close()
    p1.terminate()
    p2.terminate()
    try:
        domain = str(permission).split(";")[1]
        # options are: vibrate - getCurrentPosition - watchPosition - getUserMedia
        # a dummy log 'DISCMWEBAPILOGP' is written to avoid next accpeting conditions on calls already processed
        # if the dummy log is detected nothing happens
        if domain == "vibrate":
            logprint = "adb -s " + serial + " shell log -p v -t Dummylog \"DISCMWEBAPILOGP;nothing\""
            subprocess.run(logprint.split(" "))
            return "vibrate"
        elif domain == "getCurrentPosition" or domain == "watchPosition":
            logprint = "adb -s "+ serial +" shell log -p v -t Dummylog \"DISCMWEBAPILOGP;nothing\""
            subprocess.run(logprint.split(" "))
            return "position"
        elif domain == "getUserMedia":
            logprint = "adb -s " + serial +" shell log -p v -t Dummylog \"DISCMWEBAPILOGP;nothing\""
            subprocess.run(logprint.split(" "))
            return "media"
        else:
            return "nothing"
    except Exception as e:
        print("Can't retrieve any permission protected WEBAPI call,  "+str(e))
        return "nothing"

def checkLoading():
    p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["grep", "-a","FULLYLOADED"], stdin=p1.stdout, stdout=subprocess.PIPE)
    loaded = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
    p1.stdout.close()
    p2.stdout.close()
    p1.terminate()
    p2.terminate()
    try:
        l = str(loaded)
        if "FULLYLOADED" in l:
            return True
        else:
            return False

    except Exception as e:
        return False


def handlePermissions():
    global pos
    global vib
    
    perm = checkPermissionRequests()
    print("--------> " + perm)
    if perm  == "vibrate":
        time.sleep(0.5)
        if model == 'opo':
            sendTouchOPO(970, 400)
            sendTouchOPO(970, 460)
            sendTouchOPO(970, 520)
            sendTouchOPO(970, 600)
            sendTouchOPO(970, 700)
            sendTouchOPO(970, 800)
            sendTouchOPO(970, 900)
        else:
            sendTouchNex(920, 400)
            sendTouchNex(920, 460)
            sendTouchNex(920, 520)
            sendTouchNex(920, 550)
            sendTouchNex(920, 600)
        vib=True
        print("PERMISSION vibrate accepted") 
    elif perm == "position":
        time.sleep(0.5)
        if model == 'opo':
            sendTouchOPO(980, 330)
            sendTouchOPO(980, 430)
            sendTouchOPO(980, 530)
            sendTouchOPO(980, 630)
            sendTouchOPO(980, 730)
            sendTouchOPO(980, 800)
            sendTouchOPO(970, 900)
        else:
            sendTouchNex(920, 330)
            sendTouchNex(920, 430)
            sendTouchNex(920, 530)
            sendTouchNex(920, 630)
            sendTouchNex(920, 730)
        pos=True
        print("PERMISSION position accepted")
    elif perm == "media":
        time.sleep(0.5)
        if model == 'opo':
            sendTouchOPO(980, 440)
            sendTouchOPO(980, 540)
            sendTouchOPO(980, 640)
            sendTouchOPO(980, 740)
            sendTouchOPO(980, 840)
            sendTouchOPO(980, 940)
            sendTouchOPO(980, 1040)
            sendTouchOPO(980, 1240)
            sendTouchOPO(980, 1340)
        else:
            sendTouchNex(920, 430)
            sendTouchNex(920, 530)
            sendTouchNex(920, 630)
            sendTouchNex(920, 730)
            sendTouchNex(920, 830)
            sendTouchNex(920, 930)
            sendTouchNex(920, 1030)
            sendTouchNex(920, 1130)
            sendTouchNex(920, 1230)
            sendTouchNex(920, 1330)

        print("PERMISSION media accepted")
    else:
        print("NO PERMISSION REQUESTED")



def goBack():
    subprocess.run(("adb -s "+serial+" shell input keyevent 4").split(" "))   # back key
    logprint = "adb -s "+serial+" shell log -p v -t Redirectionlog \"Back after: "+ initialdomain + " ---> " + currentdomain + "\""
    print ("Back after: "+ initialdomain + " ---> " + currentdomain) 
    subprocess.run(logprint.split(" "))

def rollBack():
    global initialdomain
    global currentdomain
    print("Rollback backup LOG")
    rollbacklog.write(str(datetime.datetime.now()) +" Rollback needed ---" +str(initialdomain) +" ---> " +str(currentdomain)+" ["+row[1]+"]\n")
    rollback = "cp logs"+serial+"/"+row[1]+"2.txt logs"+serial+"/"+row[1]+".txt"
    #print(str(datetime.datetime.now()) + "rollback logs")
    logrollback = "adb -s "+serial+" shell log -p v -t ROLLBACK \"init:"+str(initialdomain)+ "  curr: "+str(currentdomain)+"\""
    subprocess.run(rollback.split(" "))
    subprocess.run(logrollback.split(" "))
    currentdomain = initialdomain
    #time.sleep(1)

def randomx():
    res = random.randint(0,x)
    return res

def randomy():
    res = random.randint(0,y-nav)
    res = res + 250
    return res

def sendSwipeNex():
    
    x1 = randomx()
    x2 = randomx()
    y1 = randomy()
    y2 = randomy()

    d=math.sqrt((x2-x1)**2+(y2-y1)**2)
    if d < 150:
        steps = 2
    elif d < 280:
        steps = 2
    elif d < 550:
        steps = 2
    elif d < 1100:
        steps = 3
    else:
        steps = 4
    xt = x1
    yt = y1
    dx = int((x2-x1)/steps)
    dy = int((y2-y1)/steps)
            
    for _ in range(0,steps):
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 4 14179").split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 5 187155976").split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 25").split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 53 "+str(xt)).split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 54 "+str(yt)).split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 58 61").split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))
        xt = xt + dx
        yt = yt + dy
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 4 14179").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 5 377061433").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 -1").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))

def sendSwipeOPO():
    
    x1 = randomx()
    x2 = randomx()
    y1 = randomy()
    y2 = randomy()

    d=math.sqrt((x2-x1)**2+(y2-y1)**2)
    if d < 150:
        steps = 2
    elif d < 280:
        steps = 2
    elif d < 550:
        steps = 2
    elif d < 1100:
        steps = 3
    else:
        steps = 4
    xt = x1
    yt = y1
    dx = int((x2-x1)/steps)
    dy = int((y2-y1)/steps)
    
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 9").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 1").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 1").split(" "))


    for _ in range(0,steps):
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 53 "+str(xt)).split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 54 "+str(yt)).split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 58 188").split(" "))
        subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))
        xt = xt + dx
        yt = yt + dy
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 -1").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 0").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 0").split(" "))
    subprocess.Popen(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))

def sendTouchNex(x,y):
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 4 23927").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 5 315846714").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 102").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 53 "+str(x)).split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 54 "+str(y)).split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 58 71").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 48 11").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 49 11").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 -1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))

def sendTouchOPO(x,y):
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 153").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 53 "+str(x)).split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 54 "+str(y)).split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 58 168").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 -1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))

def sendMenuOPO():
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 156").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 53 250").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 54 2007").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 58 184").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 49 5").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 3 57 -1").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 330 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 1 325 0").split(" "))
    subprocess.run(("adb -s "+serial+" shell sendevent /dev/input/event0 0 0 0").split(" "))

def sendEvent():
    if model == 'opo':
        if random.randint(0,4) > 0:
            x1 = randomx()
            y1 = randomy()
            sendTouchOPO(x1, y1)
        else:
            sendSwipeOPO()
    else:
        if random.randint(0,4) > 0:
            x1 = randomx()
            y1 = randomy()
            sendTouchNex(x1, y1)
        else:
            sendSwipeNex()
            
def checkLocationCall():
    p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["grep", "-a","getLastKnownLocation"], stdin=p1.stdout, stdout=subprocess.PIPE)
    calll = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
    p1.stdout.close()
    p2.stdout.close()
    p1.terminate()
    p2.terminate()
    try:
        call = str(calll)
        print(call)
        if 'getLastKnownLocation' in call:
            return True
        else:
            p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
            p2 = subprocess.Popen(["grep", "-a", "\"A Geolocation request can only be fulfilled in a secure context\""], stdin=p1.stdout, stdout=subprocess.PIPE)
            errr = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
            try:
                err = str(errr)
                print("EXC2: "+err)
                if 'fulfilled' in err:
                    return True
            except Exception as e:
                return False
            return False
    except Exception as e:
        print("EXC: "+str(e))
        return False

def checkVibrationCall():
    p1 = subprocess.Popen(["cat", "logs"+serial+"/" + row[1] + ".txt"], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["grep", "-a","\"android.os.SystemVibrator -Function: vibrate\""], stdin=p1.stdout, stdout=subprocess.PIPE)
    calll = subprocess.check_output(["tail", "-n", "1"], stdin=p2.stdout)
    p1.stdout.close()
    p2.stdout.close()
    p1.terminate()
    p2.terminate()
    try:
        call = str(calll)
        if 'SystemVibrator' in call:
            return True
        else:
            return False
    except Exception as e:
        return False


           
# -- SCRIPT START -- ####################################

# starts adb as root
subprocess.run(rootadb.split(" "))

global pos 
pos =False
global vib
vib = False

start_time = time.time() # initialize time, whatever it is
it = 0  #counter for iteration so restart firefox to avoid crashes

        
with open(path) as csvfile:

    reader = csv.reader(csvfile)
    for row in reader:
        if i >= int(firstsite):
            try:
                a = isSafe(row[1]) 
            except:
                print('EXCEPTION in Google SafeBrowsing request')
                a = False

            print("PREVIOUS WEBSITE VISITED FOR: "+str(time.time() - start_time)+"s")
            start_time = time.time()

            if not a:   #check if website is in the list of unsafe websites. If unsafe keeps log and nothing else happens
                
                # clean logcat checking it is actually cleaned
                m = 0
                result = False
                while m < 20 and not result:
                    result = cleanLogcat()
                    if (result):
                        print ("logcat cleaned")
                    else:
                        m = m + 1
                        print ("ERROR cleaning logcat, retrying ... (attempt: " +str(m)+")"+" ["+row[1]+"]")
                        errlog.write(str(datetime.datetime.now()) +" ERROR cleaning logcat, retrying ... (attempt: " +str(m)+")"+" ["+row[1]+"]\n")
                        time.sleep(2)
                
                if m >= 20:
                    errlog.write(str(datetime.datetime.now()) + " failed cleaning logcat after 20 attempts ["+row[1]+"]") #program goes on and error is logged 
                
                global initialdomain
                global currentdomain
                initialdomain = None
                currentdomain = None

                #subprocess.run(safebaron.split(" "))

                
                proc = None

                with open("logs"+serial+"/" + row[1]  + ".txt","ab+") as out:  # generates the logfile for the current website

                    proc = subprocess.Popen(logcommand.split(" ") ,stdout=out) 
                
                subprocess.run(("mkdir jsfiles/"+str(row[0])+row[1]).split(" "))  # generates directory containing JS files related to the current website
                
                # --- LAUNCH WEBSITE --- #

                command = bashff + row[1] + "'"
                
                subprocess.run(command.split(" "))
                
                
                # --- CHECK IF IT IS 100% LOADED OR NOT --- #
                
                end_time = time.time() + 10 # set a 10seconds timer
                
                while time.time() < end_time and not checkLoading(): # wait for at most 10 seconds that the website fully loads
                    pass
                
                if time.time() < end_time:
                    print("OK - 100% LOADED")
                else:
                    print("NOT 100% LOADED")
                    loadingfailslog.write(str(datetime.datetime.now()) +" WARNING: website is probably not 100% loaded;" + row[1] + "\n")


                # --- READ INITIAL DOMAIN OF THE WEBSITE --- #

                initialdomain = checkCurrentDomain()
                currentdomain = initialdomain

                timeout = time.time() + 25

                if str(initialdomain) is not "None" :
                    for step in range(0,60):
                        if time.time() < timeout:
                            print (" ") # space between iterations in the log
                            try:
                                print(str(initialdomain) + "  " + str(currentdomain))
                                if currentdomain in initialdomain or initialdomain in currentdomain:
                                    
                                    time.sleep(0.2)

                                    handlePermissions() 
                                    
                                    createcopy = ("cp logs"+serial+"/"+row[1]+".txt logs"+serial+"/"+row[1]+"2.txt")
                                    subprocess.run(createcopy.split(" "))
        
                                    sendEvent()
                                    
                                    time.sleep(0.2)
                                    currentdomain = checkCurrentDomain()
                                else:
                                   
                                    goBack()
                                    print("ROLLBACK1")
                                    rollBack()
                                    
                                    time.sleep(0.7)

                                    currentdomain = checkCurrentDomain()     
                                    
                                    if currentdomain in initialdomain or initialdomain in currentdomain:
                                        
                                        #print(str(datetime.datetime.now()) + "saved logs")
                                        createcopy = ("cp logs"+serial+"/"+row[1]+".txt logs"+serial+"/"+row[1]+"2.txt")
                                        subprocess.run(createcopy.split(" "))
                                         
                                        time.sleep(1)
                                        
                                        handlePermissions()
                                        
                                        sendEvent()
                                        
                                        time.sleep(0.3)

                                        currentdomain = checkCurrentDomain()
                                    else:   # if after going back the domain is still different reload the initial website
                                        subprocess.run(command.split(" "))
                                        time.sleep(2)
                                        
                                        rollBack()
                                         
                                        dummyred = "adb -s " + serial + " shell log -p v -t Dummylog \"DUMMY REDIRECTIONTO: " + str(initialdomain) + "\"" #dummy redirection log to avoid that a slow loading compromise the program (URL in the addess bar is not written immediatly)
                                        subprocess.run(dummyred.split(" "))
                                         
                                        time.sleep(0.7)
                                        
                                        handlePermissions()
                    
                                        sendEvent()
                                        
                                        time.sleep(0.3)

                                        currentdomain = checkCurrentDomain()

                                if currentdomain in initialdomain or initialdomain in currentdomain:
                                    cleanbackup = "rm logs"+serial+"/"+row[1]+"2.txt"
                                    subprocess.run(cleanbackup.split(" "))
                                else: #rollback needed so copy back the backup log to the current one

                                    goBack()

                                    time.sleep(1)
                                    curdom = checkCurrentDomain()
                                    if curdom not in initialdomain and initialdomain not in curdom:
                                        subprocess.run(command.split(" "))
                                        print("ROLLBACK2")
                                        rollBack()
                                        dummyred = "adb -s " + serial + " shell log -p v -t Dummylog \"DUMMY REDIRECTIONTO:" + str(initialdomain) + "\""
                                        subprocess.run(dummyred.split(" "))
                                        time.sleep(3)
                                    else:
                                        print("ROLLBACK3")
                                        rollBack()
                                    
                                    cleanbackup = "rm logs"+serial+"/"+row[1]+"2.txt"
                                    subprocess.run(cleanbackup.split(" "))


                            except Exception as e:
                                errlog.write(str(datetime.datetime.now()) +" MAINERROR: initdom="+str(initialdomain)+" currdom:"+str(currentdomain)+" --- " + str(e) + " ["+ row[1]  +"]\n")
                                #subprocess.run("adb shell input tap 200 1400".split(" "))
                                time.sleep(0.2)
                                if str(initialdomain) == "None":
                                    initialdomain = checkCurrentDomain() 
                                    currentdomain = initialdomain 
                                else:
                                    currentdomain = checkCurrentDomain()
                        else:
                            print( " ### TIMEOUT ### ")    # timeout reached, the number of injected events is printed
                            print("Injected "+str(step)+" events")
                            break
                else:
                    print("ERROR: can't retrieve current domain, probably failed loading website")
                    loadingfailslog.write(str(datetime.datetime.now()) +" ERROR: can't retrieve current domain, probably failed loading website;" + row[1] + "\n")
                
                
                if pos and not checkLocationCall():
                    failedgrantingpos.write(str((i*4)+1)+","+row[1]+"\n")
                if vib and not checkVibrationCall():
                    failedgrantingvib.write(str((i*4)+1)+","+row[1]+"\n")

                pos = False
                vib = False

                # download all the scripts containing interesting calls
                dl = "./scriptsdownloader.sh logs"+serial+"/"+row[1]+".txt" +" "+row[1]
                print(dl)
                
                subprocess.Popen(dl.split(" "))

                domainold = checkCurrentDomain() # update olddomain saving the last domain saved
                
                print("PID: " + str(proc.pid))
                proc.terminate()
                print("VISITED WEBSITE: "+row[1]+" NUMBER: " + str((i*4)+1))
                resetview = bashff + "localhost'"
                subprocess.run(resetview.split(" "))

                if model == 'opo':
                    if it >= 200:
                        it = 0
                        sendTouchOPO(1000, 155) # open firefox menu
                        sendTouchOPO(400,1740)  # 'Quit Firefox'
                        time.sleep(8)
                    else:
                        sendTouchOPO(900,163)   # open tabs view
                        sendTouchOPO(1024,152)  # open tabs menu
                        sendTouchOPO(634,288)   # 'close all'
                        sendTouchOPO(1000, 155) # open firefox menu
                        sendTouchOPO(656,1437)  # settings
                        sendTouchOPO(240,1869)  # clear private data
                        sendTouchOPO(773,1760)  # clean all

                        subprocess.run(killfirefox.split())
                        sendMenuOPO()           # open multitasking view
                        subprocess.run(("adb -s "+serial+" shell input swipe 600 200 600 1500 50").split(" "))   # croll view of the multitasking menu 
                        sendTouchOPO(950,160)   # kill all

                        it = it + 1
                else:

                    if it >= 200:
                        it = 0
                        sendTouchNex(1010, 140)
                        sendTouchNex(500, 1500)
                        time.sleep(8)
                    else:    
                        sendTouchNex(900,160)
                        sendTouchNex(1024,150)
                        sendTouchNex(630,280)
                        sendTouchNex(1010,140)
                        sendTouchNex(600,1270)
                        sendTouchNex(220,1650)
                        sendTouchNex(825,1680)
                        subprocess.run(killfirefox.split())
                        
                        sendTouchNex(855,1850)
                        if it%5 == 0:
                            subprocess.run(("adb -s "+serial+" shell input swipe 600 200 600 1500 50").split(" "))   # scroll view of the multitasking menu
                        sendTouchNex(950,150)
                            
                        it = it + 1

            else: # website is unsafe

                unsafewebsites.write(str(datetime.datetime.now()) + "WEBSITE CONSIDERED UNSAFE FROM GOOGLE: ["+row[1]+"]") 
        i = i + 1

